#include <FK/FK.h>
#include "glext.h"


class MyEffectModel : public fk_Model {
public:
	void preShader()
	{
		glBlendFunc(GL_ONE, GL_ONE);
		glDepthMask(GL_FALSE);
		//glDisable(GL_DEPTH_TEST);
	};
	void postShader()
	{
		//glEnable(GL_DEPTH_TEST);
		glDepthMask(GL_TRUE);
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	};
};

class fk_PointSpriteModel : public fk_Model {
private:
	fk_Point	dummy;
	fk_Image	*tex_img;
	fk_TexID	tex;

	vector<fk_FVector>	*ref_point;

public:

	fk_PointSpriteModel(void)
	{
		tex_img = NULL;
		ref_point = NULL;
		setShape(&dummy);
	};
	
	void setImage(fk_Image *argImg)
	{
		fk_ImType		*imageBuf;
		fk_Dimension	*bufSize;

		Fl::check();

		bufSize = (fk_Dimension *)argImg->getBufferSize();
		imageBuf = (fk_ImType *)argImg->getBufPointer();

		glPixelStorei(GL_UNPACK_ALIGNMENT, 4);

		// eNX`
		glGenTextures(1, &tex);
		glBindTexture(GL_TEXTURE_2D, tex);

		// eNX`gEk@̎w
		glTexParameteri(GL_TEXTURE_2D, GL_GENERATE_MIPMAP, GL_TRUE);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
		// eNX`̌JԂ@̎w
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
		// eNX`
		glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
		glTexEnvi(GL_POINT_SPRITE, GL_COORD_REPLACE, GL_TRUE);

		// eNX`]
		glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, bufSize->w, bufSize->h,
					0, GL_RGBA, GL_UNSIGNED_BYTE, imageBuf);

		tex_img = argImg;
	};

	void setPointArray(vector<fk_FVector> *argArray)
	{
		ref_point = argArray;
	};

	void preShader(void)
	{
		if(ref_point == NULL || tex_img == NULL) return;

		/* Point Sprite Lɂ */
		glEnable(GL_TEXTURE_2D);
		glEnable(GL_POINT_SPRITE);

		glBindTexture(GL_TEXTURE_2D, tex);
		// eNX`
		glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
		glTexEnvi(GL_POINT_SPRITE, GL_COORD_REPLACE, GL_TRUE);

		/* p[eBN` */
		glDisable(GL_LIGHTING);
		glColor4d(1.0, 1.0, 1.0, 0.5);
		glPointSize((GLfloat)getSize());

		glEnableClientState(GL_VERTEX_ARRAY);
		glDisableClientState(GL_NORMAL_ARRAY);
		glDisableClientState(GL_TEXTURE_COORD_ARRAY);
		glVertexPointer(3, GL_FLOAT, 0, &ref_point->at(0));

		glDrawArrays(GL_POINTS, 0, ref_point->size());
	};

	void postShader(void)
	{
		glEnable(GL_LIGHTING);
		glDisable(GL_POINT_SPRITE);
		glDisable(GL_TEXTURE_2D);
	};

};


int main(int argc, char *argv[])
{
	Fl_Window		mainWindow(512, 512, "FK TEST");
	fk_Model		camera, blockModel, lightModel;
	fk_Light		light;
	fk_Block		block(20.0, 200.0, 20.0);
	fk_Scene		scene;
	fk_Window		window(0, 0, 512, 512);

	fk_IFSTexture		blur;
	MyEffectModel		blurModel;
	vector<fk_Vector>	blurArray;

	fk_Image			tex;
	vector<fk_FVector>	pointArray;
	fk_PointSpriteModel	ps;

	mainWindow.end();
	fk_InitMaterial();
	fk_SetErrorMode(FK_ERR_BROWSER_INTERACTIVE);

	// Ɩ̐ݒ
	lightModel.setShape(&light);
	lightModel.setMaterial(White);
	lightModel.glMoveTo(0.0, 0.0, 0.0);
	lightModel.glFocus(-1.0, -1.0, -1.0);

	// ̂̐ݒ
	blockModel.setShape(&block);
	blockModel.setMaterial(Yellow);
	blockModel.glMoveTo(0.0, 100.0, 0.0);

	// effBXvCXgɓo^
	scene.setBlendStatus(true);
	scene.setBGColor(0.5f, 0.5f, 0.5f);
	scene.entryCamera(&camera);
	scene.entryModel(&blockModel);
	scene.entryModel(&lightModel);

	// EBhEփfBXvCXgo^
	window.setScene(&scene);

	// _̈ʒuƎpݒ
	camera.glMoveTo(0.0, 0.0, 1000.0);
	camera.glFocus(0.0, 0.0, 0.0);
	camera.glUpvec(0.0, 1.0, 0.0);

	mainWindow.show();
	window.show();

	if(!tex.readBMP("tex.bmp")) fl_alert("ps failed.");
	for(int i = 0; i < 10; i++) {
		pointArray.push_back(fk_Vector(rand()%500 - 250.0, rand()%500 - 250.0, rand()%500 - 250.0));
	}
	ps.setPointArray(&pointArray);
	ps.setImage(&tex);
	ps.setSize(15.0);

	scene.entryModel(&ps);

	if(!blur.readBMP("blur.bmp")) fl_alert("bmp failed.");
	if(!blur.readMQOFile("blur.mqo", "obj1")) fl_alert("mqo failed.");
	blur.setTextureMode(FK_TEX_REPLACE);
	blur.setTexRendMode(FK_TEX_REND_SMOOTH);
	blurModel.setShape(&blur);
	blurArray.clear();

	while(true) {

		if(mainWindow.visible() == 0) {
			if(Fl::wait() == 0) {
				break;
			} else {
				continue;
			}
		}

		if(window.drawWindow() == 0) break;
		if(Fl::check() == 0) break;
		if(window.winOpenStatus() == false) continue;

		// u@@@@@@@@
		if(window.getKeyStatus(' ', false)) {
			scene.entryModel(&blurModel);
			blurArray.push_back(blockModel.getMatrix()*fk_Vector(0.0, -50.0, 0.0));
			blurArray.push_back(blockModel.getMatrix()*fk_Vector(0.0, 150.0, 0.0));
			if(blurArray.size() > 16){
				blurArray.erase(blurArray.begin());
				blurArray.erase(blurArray.begin());
			}

			// ̂ Y Sɉ]
			blockModel.glRotateWithVec(0.0, 0.0, 0.0, fk_Z, FK_PI/10.0);
		} else {
			if(blurArray.size() > 0){
				blurArray.erase(blurArray.begin());
				blurArray.erase(blurArray.begin());
			} else {
				blurArray.clear();
				scene.removeModel(&blurModel);
			}
		}

		unsigned int k = blurArray.size();
		for(unsigned int j = 0; j < 9; j++) {
			if(k == 0) {
				blur.getIFS()->moveVPosition(j*2, blockModel.getMatrix()*fk_Vector(0.0, -50.0, 0.0));
				blur.getIFS()->moveVPosition(j*2+1, blockModel.getMatrix()*fk_Vector(0.0, 150.0, 0.0));
				continue;
			}
			blur.getIFS()->moveVPosition(j*2, blurArray[k-2]);
			blur.getIFS()->moveVPosition(j*2+1, blurArray[k-1]);
			k -= 2;
		}
	}

	return 0;
}
